class Query<TItem>{
    get quantidade():number{
        return this.itens.length;
    }
    get primeiro():TItem{
        if(this.quantidade > 0)
            return this.itens[0];
        else
            return null;
    }
    get ultimo():TItem{
        if(this.quantidade > 0)
            return this.itens[this.quantidade-1];
        else
            return null;        
    }

    constructor(public itens: TItem[]){ 
        if(!this.itens)
            this.itens = [];
    }
    quando(selecao: (item:TItem, indice: number) => boolean): Query<TItem>{
        var _retorno = [];
        for (var r = 0; r < this.itens.length; r++) {
            if (selecao(this.itens[r], r) == true)
                _retorno.push(this.itens[r]);
        }
        return new Query<TItem>(_retorno);
    }
    converte<TRetorno>(conversao: (item: TItem, indice: number) => TRetorno):Query<TRetorno>{
        var _retorno = [];
        for (var r = 0; r < this.itens.length; r++) {
            var _itemAdd = conversao(this.itens[r], r);
            _retorno.push(_itemAdd);
        }
        return new Query<TRetorno>(_retorno);        
    }
    contem(selecao: (item: TItem, indice: number) => boolean):boolean{
        for (var r = 0; r < this.itens.length; r++) {
            if (selecao(this.itens[r], r) == true)
                return true;
        }
        return false;
    }
    ocorrencias(selecao: (item: TItem, indice: number) => boolean): Query<number>{
        var _retorno = new Query<number>([]);
        for (var r = 0; r < this.itens.length; r++) {
            if(selecao(this.itens[r], r))
                _retorno.itens.push(r);
        }
        return _retorno;
    }    
    distingue(igualdade: (esquerda: TItem, direita: TItem) => boolean) :Query<TItem>{
        var _retorno = new Query<TItem>([]);
        for (var r = 0; r < this.itens.length; r++) {
            var _itemEsquerda = this.itens[r];
            if(!_retorno.contem(itemDireita => igualdade(_itemEsquerda, itemDireita)))
                _retorno.itens.push(_itemEsquerda);
        }
        return _retorno;        
    }
    distinguePropiedade(nomePropriedade: string) :Query<TItem>{
        var _igualdade = (esquerda, direita) => esquerda[nomePropriedade] == direita[nomePropriedade];
        return this.distingue(_igualdade);
    }
    ordena(ordenacao: (esquerda: TItem, direita: TItem)=>number): Query<TItem>{
        /*
        var points = [40, 100, 1, 5, 25, 10];
        points.sort(function(a, b){return a-b});        
        */
        return new Query<TItem>(this.itens.sort(ordenacao));
    }
    agrupa<TGrupo>(agrupamento: (item: TItem, grupos:Query<TGrupo>) => TGrupo): Query<TGrupo>{
        var _retorno = new Query<TGrupo>([]);
        for (var r = 0; r < this.itens.length; r++) {
            var _grupo = agrupamento(this.itens[r], _retorno);
            if(_grupo && (_grupo != null))
                _retorno.itens.push(_grupo);
        }
        return _retorno;
    }
    junta(lista:TItem[]):Query<TItem>{
        for (let r = 0; r < lista.length; r++) {
            const itemAdd = lista[r];
            this.itens.push(itemAdd);
        }
        return this;
    }
    adiciona(item: TItem):TItem{
        this.itens.push(item);
        return item;
    }    
}

export = Query;

//tsc Query.ts --module CommonJS --target ES5